/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;

using CookComputing.XmlRpc;


namespace XenAPI
{
    public partial class Event : XenObject<Event>
    {
        public Event()
        {
        }

        internal Event(Proxy_Event proxy)
        {
            this.UpdateFromProxy(proxy);
        }

        internal void UpdateFromProxy(Proxy_Event proxy)
        {
            id = long.Parse((string)proxy.id);
        }

        public override void UpdateFrom(Event update)
        {
            id = update.id;
        }

        internal Proxy_Event ToProxy()
        {
            Proxy_Event result = new Proxy_Event();
            result.id = id.ToString();
            return result;
        }

        public override string SaveChanges(Session session, string opaqueRef, Event serverObject)
        {
            Event server = (Event)serverObject;
            if (opaqueRef == null)
            {
                Proxy_Event p = this.ToProxy();
                throw new InvalidOperationException("There is no constructor available for this type; you cannot directly create one on the server.");
            }
            else
            {
                if (!_id.Equals(server._id))
                {
                    Event.set_id(session, opaqueRef, _id);
                }

                return null;
            }
        }

        public static Event get_record(Session session, string _event)
        {
            return new Event((Proxy_Event)session.proxy.event_get_record(session.uuid, (_event != null) ? _event : "").parse());
        }

        public static string get_by_uuid(Session session, string _uuid)
        {
            return (string)session.proxy.event_get_by_uuid(session.uuid, (_uuid != null) ? _uuid : "").parse();
        }

        public static long get_id(Session session, string _event)
        {
            return long.Parse((string)session.proxy.event_get_id(session.uuid, (_event != null) ? _event : "").parse());
        }

        public static void set_id(Session session, string _event, long _id)
        {
            session.proxy.event_set_id(session.uuid, (_event != null) ? _event : "", _id.ToString()).parse();
        }

        public static void register(Session session, string[] _classes)
        {
            session.proxy.event_register(session.uuid, _classes).parse();
        }

        public static void unregister(Session session, string[] _classes)
        {
            session.proxy.event_unregister(session.uuid, _classes).parse();
        }

        public static Proxy_Event[] next(Session session)
        {
            return session.proxy.event_next(session.uuid).parse();
        }

        private long _id;
        public long id
        {
             get { return _id; }
             set { if (value != _id) { _id = value; Changed = true; NotifyPropertyChanged("id"); } }
        }
    }
}
