/*
 * Copyright (c) Citrix Systems, Inc. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;

namespace XenAPI
{
    public partial class Relation
    {
        public readonly String field;
        public readonly String manyType;
        public readonly String manyField;

        public Relation(String field, String manyType, String manyField)
        {
            this.field = field;
            this.manyField = manyField;
            this.manyType = manyType;
        }

        public static Dictionary<Type, Relation[]> GetRelations()
        {
            Dictionary<Type, Relation[]> relations = new Dictionary<Type, Relation[]>();

            relations.Add(typeof(Proxy_Role), new Relation[] {
                new Relation("subroles", "role", "subroles"),
            });

            relations.Add(typeof(Proxy_Network), new Relation[] {
                new Relation("PIFs", "PIF", "network"),
                new Relation("VIFs", "VIF", "network"),
            });

            relations.Add(typeof(Proxy_VMPP), new Relation[] {
                new Relation("VMs", "VM", "protection_policy"),
            });

            relations.Add(typeof(Proxy_VDI), new Relation[] {
                new Relation("crash_dumps", "crashdump", "VDI"),
                new Relation("VBDs", "VBD", "VDI"),
                new Relation("snapshots", "VDI", "snapshot_of"),
            });

            relations.Add(typeof(Proxy_VM), new Relation[] {
                new Relation("attached_PCIs", "PCI", "attached_VMs"),
                new Relation("VGPUs", "VGPU", "VM"),
                new Relation("consoles", "console", "VM"),
                new Relation("VTPMs", "VTPM", "VM"),
                new Relation("VIFs", "VIF", "VM"),
                new Relation("crash_dumps", "crashdump", "VM"),
                new Relation("VBDs", "VBD", "VM"),
                new Relation("children", "VM", "parent"),
                new Relation("snapshots", "VM", "snapshot_of"),
            });

            relations.Add(typeof(Proxy_DR_task), new Relation[] {
                new Relation("introduced_SRs", "SR", "introduced_by"),
            });

            relations.Add(typeof(Proxy_VM_appliance), new Relation[] {
                new Relation("VMs", "VM", "appliance"),
            });

            relations.Add(typeof(Proxy_Task), new Relation[] {
                new Relation("subtasks", "task", "subtask_of"),
            });

            relations.Add(typeof(Proxy_GPU_group), new Relation[] {
                new Relation("VGPUs", "VGPU", "GPU_group"),
                new Relation("PGPUs", "PGPU", "GPU_group"),
            });

            relations.Add(typeof(Proxy_Bond), new Relation[] {
                new Relation("slaves", "PIF", "bond_slave_of"),
            });

            relations.Add(typeof(Proxy_Pool), new Relation[] {
                new Relation("metadata_VDIs", "VDI", "metadata_of_pool"),
            });

            relations.Add(typeof(Proxy_PIF), new Relation[] {
                new Relation("tunnel_transport_PIF_of", "tunnel", "transport_PIF"),
                new Relation("tunnel_access_PIF_of", "tunnel", "access_PIF"),
                new Relation("VLAN_slave_of", "VLAN", "tagged_PIF"),
                new Relation("bond_master_of", "Bond", "master"),
            });

            relations.Add(typeof(Proxy_Subject), new Relation[] {
                new Relation("roles", "subject", "roles"),
            });

            relations.Add(typeof(Proxy_Host), new Relation[] {
                new Relation("PGPUs", "PGPU", "host"),
                new Relation("PCIs", "PCI", "host"),
                new Relation("patches", "host_patch", "host"),
                new Relation("crashdumps", "host_crashdump", "host"),
                new Relation("host_CPUs", "host_cpu", "host"),
                new Relation("resident_VMs", "VM", "resident_on"),
                new Relation("PIFs", "PIF", "host"),
                new Relation("PBDs", "PBD", "host"),
            });

            relations.Add(typeof(Proxy_Session), new Relation[] {
                new Relation("tasks", "task", "session"),
            });

            relations.Add(typeof(Proxy_Pool_patch), new Relation[] {
                new Relation("host_patches", "host_patch", "pool_patch"),
            });

            relations.Add(typeof(Proxy_SR), new Relation[] {
                new Relation("VDIs", "VDI", "SR"),
                new Relation("PBDs", "PBD", "SR"),
            });


            return relations;
       }
    }
}
