/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace XenAPI
{
    public abstract partial class XenObject<S> : IXenObject where S : XenObject<S>
    {
        /// <summary>
        /// Copies properties from 'update' into this object. It uses property setters so that
        /// event handlers are triggered if the value is changing.
        /// </summary>
        public abstract void UpdateFrom(S update);

        /// <summary>
        /// Save any changed fields to the server. 
        /// This method is usually invoked on a thread pool thread.
        /// </summary>
        /// <param name="serverObject">Changes are sent to the server if the field in "this" is different from serverObject.
        /// Can be the object in the cache, or another reference object that we want to save changes to.</param>
        public abstract string SaveChanges(Session session, string serverOpaqueRef, S serverObject);

        private string _opaque_ref = null;
        public string opaque_ref
        {
            get { return _opaque_ref; }
            set { _opaque_ref = value; }
        }

        private bool _changed = false;
        public bool Changed
        {
            get { return _changed; }
            set { _changed = value; }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(info));
            }
        }

        public void ClearEventListeners()
        {
            PropertyChanged = null;
        }
    }
}
